@@ -3,18 +3,21 @@
 #include <glib/gi18n.h>
 #include <glibtop/procopenfiles.h>
 
+#include <sys/wait.h>
+
+// #include <libsexy/sexy-icon-entry.h>
+
+
 #include <set>
 #include <string>
 #include <sstream>
 #include <iterator>
 
-#include <pcrecpp.h>
+// #include <pcrecpp.h>
 
-extern "C" {
 #include "procman.h"
 #include "lsof.h"
 #include "util.h"
-}
 
 
 using std::string;
@@ -84,7 +87,7 @@ namespace
     { }
   };
 
-#else
+#elif 0
 
   class Lsof
     : public LsofBase
@@ -103,6 +106,74 @@ namespace
     { }
   };
 
+#else
+  class Lsof
+    : public LsofBase
+  {
+    string pattern;
+    bool caseless;
+
+
+    static string escape(const string &s)
+    {
+      char *escaped = g_strescape(s.c_str(), "");
+      string ret(escaped);
+      g_free(escaped);
+
+      return ret;
+    }
+
+
+    virtual bool matches(const string &filename) const
+    {
+      string argv1, argv2;
+
+      argv1 = escape(this->pattern);
+      argv2 = escape(filename);
+
+      const char *argv[7];
+
+      argv[0] = "python";
+      argv[1] = "-c";
+      argv[2] = "import sys, re; "
+	"sys.exit(re.search(sys.argv[1], sys.argv[2], "
+	"(bool(sys.argv[3]) and re.I or 0)) is None)";
+      argv[3] = argv1.c_str();
+      argv[4] = argv2.c_str();
+      argv[5] = this->caseless ? "1" : "0";
+      argv[6] = NULL;
+
+      int status;
+      GError *error = NULL;
+
+      if (g_spawn_sync(NULL, // cwd
+		       const_cast<gchar**>(argv), NULL, // argv and env
+		       static_cast<GSpawnFlags>(G_SPAWN_SEARCH_PATH
+					       | G_SPAWN_STDOUT_TO_DEV_NULL
+					       | G_SPAWN_STDERR_TO_DEV_NULL), // flags
+		       NULL, NULL, // child_setup and user_data
+		       NULL, NULL, // stdin, stdout
+		       &status,
+		       &error)) {
+	if (!error and WIFEXITED(status))
+	  return WEXITSTATUS(status) == 0;
+      }
+
+      if (error) {
+	procman_debug("Failed to spawn python for re : %s", error->message);
+	g_error_free(error);
+      }
+
+      return false;
+    }
+
+  public:
+
+    Lsof(const string &pattern, bool caseless)
+      : pattern(pattern), caseless(caseless)
+    { }
+  };
+
 #endif
 
 
@@ -208,6 +279,12 @@ namespace
     }
 
 
+    static void search_entry_activate(GtkEntry *, gpointer data)
+    {
+      static_cast<GUI*>(data)->search();
+    }
+
+
     static void clear_button_clicked(GtkButton *, gpointer data)
     {
       static_cast<GUI*>(data)->clear();
@@ -345,15 +422,18 @@ void procman_lsof(ProcData *procdata)
   GtkWidget *label = gtk_label_new_with_mnemonic(_("_Name contains:"));
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   GtkWidget *entry = gtk_entry_new();
+
+  // entry = sexy_icon_entry_new();
+  // sexy_icon_entry_add_clear_button(SEXY_ICON_ENTRY(entry));
+  // GtkWidget *icon = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
+  // sexy_icon_entry_set_icon(SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_PRIMARY, GTK_IMAGE(icon));
+
   gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
   GtkWidget *search_button = gtk_button_new_from_stock(GTK_STOCK_FIND);
   gtk_box_pack_start(GTK_BOX(hbox), search_button, FALSE, FALSE, 0);
   GtkWidget *clear_button = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
   gtk_box_pack_start(GTK_BOX(hbox), clear_button, FALSE, FALSE, 0);
 
-  // g_object_set(G_OBJECT(dialog), "activates_default", TRUE, NULL);
-  // gtk_window_set_default(GTK_WINDOW(dialog), GTK_WIDGET(search_button));
-
 
   GtkWidget *case_button = gtk_check_button_new_with_mnemonic(_("Case insensitive matching"));
   GtkWidget *hbox3 = gtk_hbox_new(FALSE, 12);
@@ -394,6 +474,8 @@ void procman_lsof(ProcData *procdata)
   gui->entry = GTK_ENTRY(entry);
   gui->count = GTK_LABEL(count_label);
 
+  g_signal_connect(G_OBJECT(entry), "activate",
+		   G_CALLBACK(GUI::search_entry_activate), gui);
   g_signal_connect(G_OBJECT(clear_button), "clicked",
 		   G_CALLBACK(GUI::clear_button_clicked), gui);
   g_signal_connect(G_OBJECT(search_button), "clicked",
